package gregapi.cover.covers;

import static gregapi.data.CS.*;
import gregapi.cover.CoverData;
import gregapi.data.LH;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.ITexture;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidBlock;
import net.minecraftforge.fluids.IFluidHandler;

/**
 * @author Gregorius Techneticies
 */
public class CoverDrain extends AbstractCoverAttachment {
	@Override public boolean interceptCoverPlacement(byte aCoverSide, CoverData aData, Entity aPlayer) {return !(aData.mTileEntity.canTick() && aData.mTileEntity instanceof IFluidHandler);}
	
	@Override
	public void onTickPre(byte aSide, CoverData aData, long aTimer, boolean aIsServerSide, boolean aReceivedBlockUpdate, boolean aReceivedInventoryUpdate) {
		if (aIsServerSide && aData.mTileEntity instanceof IFluidHandler) {
			if (SIDES_TOP_HORIZONTAL[aSide] && aTimer % 100 == 6 && aData.mTileEntity.getWorld().isRaining() && !UT.Worlds.hasBlockHitBox(aData.mTileEntity.getWorld(), aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide))) {
				BiomeGenBase tBiome = aData.mTileEntity.getBiome();
				if (tBiome.rainfall > 0 && tBiome.temperature >= 0.2 && aData.mTileEntity.getRainAtSide(aSide)) {
					if (SIDES_TOP[aSide] || aData.mTileEntity.getBlockOffset(OFFSETS_X[aSide], -1, OFFSETS_Z[aSide]).isSideSolid(aData.mTileEntity.getWorld(), aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getY()-1, aData.mTileEntity.getOffsetZ(aSide), FORGE_DIR[SIDE_TOP])) {
						FluidStack tWater = UT.Fluids.water((long)Math.max(1, tBiome.rainfall*100) * (aData.mTileEntity.getWorld().isThundering()?2:1));
						if (((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[aSide], tWater, T) == 0) ((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[SIDE_ANY], tWater, T);
					}
				}
			}
			if (aReceivedBlockUpdate || aTimer % 20 == 5) {
				Block tBlock = aData.mTileEntity.getBlockAtSide(aSide);
				FluidStack tFluid = NF;
				if (tBlock == Blocks.water || tBlock == Blocks.flowing_water) if (aData.mTileEntity.getMetaDataAtSide(aSide) == 0) tFluid = UT.Fluids.water(1000);
				if (tBlock == Blocks.lava  || tBlock == Blocks.flowing_lava ) if (aData.mTileEntity.getMetaDataAtSide(aSide) == 0) tFluid = UT.Fluids.lava (1000);
				if (tBlock instanceof IFluidBlock) tFluid = ((IFluidBlock)tBlock).drain(aData.mTileEntity.getWorld(), aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide), F);
				
				if (tFluid != null && (SIDES_HORIZONTAL[aSide] || UT.Fluids.gas(tFluid) || (tFluid.getFluid().getDensity(tFluid)<0?SIDES_BOTTOM:SIDES_TOP)[aSide])) {
					if (((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[aSide], tFluid, F) == tFluid.amount) {
						((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[aSide], tFluid, T);
						if (tBlock instanceof IFluidBlock) {
							((IFluidBlock)tBlock).drain(aData.mTileEntity.getWorld(), aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide), T);
						} else {
							aData.mTileEntity.getWorld().setBlockToAir(aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide));
						}
					} else
					if (((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[SIDE_ANY], tFluid, F) == tFluid.amount) {
						((IFluidHandler)aData.mTileEntity).fill(FORGE_DIR[SIDE_ANY], tFluid, T);
						if (tBlock instanceof IFluidBlock) {
							((IFluidBlock)tBlock).drain(aData.mTileEntity.getWorld(), aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide), T);
						} else {
							aData.mTileEntity.getWorld().setBlockToAir(aData.mTileEntity.getOffsetX(aSide), aData.mTileEntity.getOffsetY(aSide), aData.mTileEntity.getOffsetZ(aSide));
						}
					}
				}
			}
		}
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		super.addToolTips(aList, aStack, aF3_H);
		aList.add(LH.Chat.CYAN + "Collects Fluid Blocks (if not against Gravity)");
		aList.add(LH.Chat.CYAN + "Collects Rainwater (not in Deserts or Cold Areas)");
	}
	
	@Override public ITexture getCoverTextureSurface(byte aSide, CoverData aData) {return sTextureFront;}
	@Override public ITexture getCoverTextureAttachment(byte aSide, CoverData aData, byte aTextureSide) {return aSide == aTextureSide ? sTextureFront : aSide == OPPOSITES[aTextureSide] ? sTextureBack : sTextureSides;}
	@Override public ITexture getCoverTextureHolder(byte aSide, CoverData aData, byte aTextureSide) {return sTextureSides;}
	
	public static final ITexture
	sTextureSides = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/drain/sides")),
	sTextureFront = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/drain/front")),
	sTextureBack = BlockTextureDefault.get(new Textures.BlockIcons.CustomIcon("machines/covers/drain/back"));
}